@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  *          Martyn Russell <martyn@imendio.com>
  */
@@ -26,28 +26,29 @@
 #define __EMPATHY_ACCOUNT_WIDGET_GENERIC_H__
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
-#include <libmissioncontrol/mc-account.h>
+#include <libempathy/empathy-account.h>
 
 G_BEGIN_DECLS
 
-void       empathy_account_widget_handle_params     (McAccount   *account,
-						     GladeXML    *glade,
-						     const gchar *first_widget_name,
+void       empathy_account_widget_handle_params     (EmpathyAccount   *account,
+						     GtkBuilder  *gui,
+						     const gchar *first_widget,
 						     ...);
-void       empathy_account_widget_add_forget_button (McAccount   *account,
-						     GladeXML    *glade,
+void       empathy_account_widget_add_forget_button (EmpathyAccount   *account,
+						     GtkBuilder  *gui,
 						     const gchar *button,
 						     const gchar *entry);
-GtkWidget *empathy_account_widget_generic_new       (McAccount   *account);
-GtkWidget *empathy_account_widget_salut_new         (McAccount   *account);
-GtkWidget *empathy_account_widget_msn_new           (McAccount   *account);
-GtkWidget *empathy_account_widget_jabber_new        (McAccount   *account);
-GtkWidget *empathy_account_widget_icq_new           (McAccount   *account);
-GtkWidget *empathy_account_widget_aim_new           (McAccount   *account);
-GtkWidget *empathy_account_widget_yahoo_new         (McAccount   *account);
-GtkWidget *empathy_account_widget_groupwise_new     (McAccount   *account);
+void	   empathy_account_widget_set_default_focus (GtkBuilder  *gui,
+						     const gchar *entry);
+GtkWidget *empathy_account_widget_generic_new       (EmpathyAccount   *account);
+GtkWidget *empathy_account_widget_salut_new         (EmpathyAccount   *account);
+GtkWidget *empathy_account_widget_msn_new           (EmpathyAccount   *account);
+GtkWidget *empathy_account_widget_jabber_new        (EmpathyAccount   *account);
+GtkWidget *empathy_account_widget_icq_new           (EmpathyAccount   *account);
+GtkWidget *empathy_account_widget_aim_new           (EmpathyAccount   *account);
+GtkWidget *empathy_account_widget_yahoo_new         (EmpathyAccount   *account);
+GtkWidget *empathy_account_widget_groupwise_new     (EmpathyAccount   *account);
 
 G_END_DECLS
 
