@@ -25,12 +25,12 @@
 #include <stdlib.h>
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <libmissioncontrol/mission-control.h>
 
 #include <libempathy/empathy-contact-manager.h>
+#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-utils.h>
 
@@ -40,9 +40,10 @@
 
 static GList *subscription_dialogs = NULL;
 static GList *information_dialogs = NULL;
+static GList *edit_dialogs = NULL;
+static GtkWidget *personal_dialog = NULL;
 static GtkWidget *new_contact_dialog = NULL;
 
-
 static gint
 contact_dialogs_find (GtkDialog      *dialog,
 		      EmpathyContact *contact)
@@ -68,7 +69,7 @@ subscription_dialog_response_cb (GtkDialog *dialog,
 	EmpathyContactManager *manager;
 	EmpathyContact        *contact;
 
-	manager = empathy_contact_manager_new ();
+	manager = empathy_contact_manager_dup_singleton ();
 	contact = empathy_contact_widget_get_contact (contact_widget);
 
 	if (response == GTK_RESPONSE_YES) {
@@ -89,6 +90,7 @@ void
 empathy_subscription_dialog_show (EmpathyContact *contact,
 				  GtkWindow     *parent)
 {
+	GtkBuilder *gui;
 	GtkWidget *dialog;
 	GtkWidget *hbox_subscription;
 	GtkWidget *contact_widget;
@@ -105,16 +107,16 @@ empathy_subscription_dialog_show (EmpathyContact *contact,
 		return;
 	}
 
-	filename = empathy_file_lookup ("empathy-contact-dialogs.glade",
+	filename = empathy_file_lookup ("empathy-contact-dialogs.ui",
 					"libempathy-gtk");
-	empathy_glade_get_file_simple (filename,
-				      "subscription_request_dialog",
-				      NULL,
+	gui = empathy_builder_get_file (filename,
 				      "subscription_request_dialog", &dialog,
 				      "hbox_subscription", &hbox_subscription,
 				      NULL);
 	g_free (filename);
+	g_object_unref (gui);
 
+	/* Contact info widget */
 	contact_widget = empathy_contact_widget_new (contact,
 						     EMPATHY_CONTACT_WIDGET_EDIT_ALIAS |
 						     EMPATHY_CONTACT_WIDGET_EDIT_GROUPS);
@@ -122,6 +124,7 @@ empathy_subscription_dialog_show (EmpathyContact *contact,
 			  contact_widget,
 			  TRUE, TRUE,
 			  0);
+	gtk_widget_show (contact_widget);
 
 	g_object_set_data (G_OBJECT (dialog), "contact_widget", contact_widget);
 	subscription_dialogs = g_list_prepend (subscription_dialogs, dialog);
@@ -142,25 +145,22 @@ empathy_subscription_dialog_show (EmpathyContact *contact,
  */
 
 static void
-contact_information_response_cb (GtkDialog *dialog,
-				 gint       response,
-				 GtkWidget *contact_widget)
+contact_dialogs_response_cb (GtkDialog *dialog,
+			     gint       response,
+			     GList    **dialogs)
 {
-	information_dialogs = g_list_remove (information_dialogs, dialog);
+	*dialogs = g_list_remove (*dialogs, dialog);
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 void
 empathy_contact_information_dialog_show (EmpathyContact *contact,
-					 GtkWindow      *parent,
-					 gboolean        edit,
-					 gboolean        is_user)
+					 GtkWindow      *parent)
 {
-	GtkWidget                *dialog;
-	GtkWidget                *button;
-	GtkWidget                *contact_widget;
-	GList                    *l;
-	EmpathyContactWidgetFlags flags = 0;
+	GtkWidget *dialog;
+	GtkWidget *button;
+	GtkWidget *contact_widget;
+	GList     *l;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
@@ -176,15 +176,7 @@ empathy_contact_information_dialog_show (EmpathyContact *contact,
 	dialog = gtk_dialog_new ();
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-	if (is_user) {
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Personal Information"));
-	}
-	else if (edit) {
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Contact Information"));
-	}
-	else {
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Contact Information"));
-	}
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Contact Information"));
 
 	/* Close button */
 	button = gtk_button_new_with_label (GTK_STOCK_CLOSE);
@@ -197,32 +189,21 @@ empathy_contact_information_dialog_show (EmpathyContact *contact,
 	gtk_widget_show (button);
 
 	/* Contact info widget */
-	if (edit) {
-		flags |= EMPATHY_CONTACT_WIDGET_EDIT_ALIAS;
-	}
-	if (is_user) {
-		flags |= EMPATHY_CONTACT_WIDGET_EDIT_ACCOUNT;
-		flags |= EMPATHY_CONTACT_WIDGET_EDIT_AVATAR;
-	}
-	if (!is_user && edit) {
-		flags |= EMPATHY_CONTACT_WIDGET_EDIT_GROUPS;
-	}
-	contact_widget = empathy_contact_widget_new (contact, flags);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	contact_widget = empathy_contact_widget_new (contact,
+		EMPATHY_CONTACT_WIDGET_SHOW_LOCATION |
+		EMPATHY_CONTACT_WIDGET_EDIT_NONE);
+	gtk_container_set_border_width (GTK_CONTAINER (contact_widget), 8);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 			    contact_widget,
 			    TRUE, TRUE, 0);
-	if (flags & EMPATHY_CONTACT_WIDGET_EDIT_ACCOUNT) {
-		empathy_contact_widget_set_account_filter (contact_widget,
-							   empathy_account_chooser_filter_is_connected,
-							   NULL);
-	}
+	gtk_widget_show (contact_widget);
 
 	g_object_set_data (G_OBJECT (dialog), "contact_widget", contact_widget);
 	information_dialogs = g_list_prepend (information_dialogs, dialog);
 
 	g_signal_connect (dialog, "response",
-			  G_CALLBACK (contact_information_response_cb),
-			  contact_widget);
+			  G_CALLBACK (contact_dialogs_response_cb),
+			  &information_dialogs);
 
 	if (parent) {
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
@@ -231,39 +212,139 @@ empathy_contact_information_dialog_show (EmpathyContact *contact,
 	gtk_widget_show (dialog);
 }
 
+void
+empathy_contact_edit_dialog_show (EmpathyContact *contact,
+				  GtkWindow      *parent)
+{
+	GtkWidget *dialog;
+	GtkWidget *button;
+	GtkWidget *contact_widget;
+	GList     *l;
+
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+	l = g_list_find_custom (edit_dialogs,
+				contact,
+				(GCompareFunc) contact_dialogs_find);
+	if (l) {
+		gtk_window_present (GTK_WINDOW (l->data));
+		return;
+	}
+
+	/* Create dialog */
+	dialog = gtk_dialog_new ();
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Contact Information"));
+
+	/* Close button */
+	button = gtk_button_new_with_label (GTK_STOCK_CLOSE);
+	gtk_button_set_use_stock (GTK_BUTTON (button), TRUE);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+				      button,
+				      GTK_RESPONSE_CLOSE);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_window_set_default (GTK_WINDOW (dialog), button);
+	gtk_widget_show (button);
+
+	/* Contact info widget */
+	contact_widget = empathy_contact_widget_new (contact,
+		EMPATHY_CONTACT_WIDGET_EDIT_ALIAS |
+		EMPATHY_CONTACT_WIDGET_EDIT_GROUPS);
+	gtk_container_set_border_width (GTK_CONTAINER (contact_widget), 8);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+			    contact_widget,
+			    TRUE, TRUE, 0);
+	gtk_widget_show (contact_widget);
+
+	g_object_set_data (G_OBJECT (dialog), "contact_widget", contact_widget);
+	edit_dialogs = g_list_prepend (edit_dialogs, dialog);
+
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (contact_dialogs_response_cb),
+			  &edit_dialogs);
+
+	if (parent) {
+		gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+	}
+
+	gtk_widget_show (dialog);
+}
+
+void
+empathy_contact_personal_dialog_show (GtkWindow *parent)
+{
+	GtkWidget *button;
+	GtkWidget *contact_widget;
+
+	if (personal_dialog) {
+		gtk_window_present (GTK_WINDOW (personal_dialog));
+		return;
+	}
+
+	/* Create dialog */
+	personal_dialog = gtk_dialog_new ();
+	gtk_dialog_set_has_separator (GTK_DIALOG (personal_dialog), FALSE);
+	gtk_window_set_resizable (GTK_WINDOW (personal_dialog), FALSE);
+	gtk_window_set_title (GTK_WINDOW (personal_dialog), _("Personal Information"));
+
+	/* Close button */
+	button = gtk_button_new_with_label (GTK_STOCK_CLOSE);
+	gtk_button_set_use_stock (GTK_BUTTON (button), TRUE);
+	gtk_dialog_add_action_widget (GTK_DIALOG (personal_dialog),
+				      button,
+				      GTK_RESPONSE_CLOSE);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_window_set_default (GTK_WINDOW (personal_dialog), button);
+	gtk_widget_show (button);
+
+	/* Contact info widget */
+	contact_widget = empathy_contact_widget_new (NULL,
+		EMPATHY_CONTACT_WIDGET_EDIT_ACCOUNT |
+		EMPATHY_CONTACT_WIDGET_EDIT_ALIAS |
+		EMPATHY_CONTACT_WIDGET_EDIT_AVATAR);
+	gtk_container_set_border_width (GTK_CONTAINER (contact_widget), 8);
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (personal_dialog))),
+			    contact_widget,
+			    TRUE, TRUE, 0);
+	empathy_contact_widget_set_account_filter (contact_widget,
+		empathy_account_chooser_filter_is_connected, NULL);
+	gtk_widget_show (contact_widget);
+
+	g_signal_connect (personal_dialog, "response",
+			  G_CALLBACK (gtk_widget_destroy), NULL);
+	g_object_add_weak_pointer (G_OBJECT (personal_dialog),
+				   (gpointer) &personal_dialog);
+
+	if (parent) {
+		gtk_window_set_transient_for (GTK_WINDOW (personal_dialog), parent);
+	}
+
+	gtk_widget_show (personal_dialog);
+}
+
 /*
  *  New contact dialog
  */
 
 static gboolean
-can_add_contact_to_account (McAccount *account,
+can_add_contact_to_account (EmpathyAccount *account,
 			    gpointer   user_data)
 {
-	MissionControl     *mc;
-	TpConnectionStatus  status;
-	McProfile          *profile;
-	const gchar        *protocol_name;
+	EmpathyContactManager *contact_manager;
+	TpConnection          *connection;
+	gboolean               result;
 
-	mc = empathy_mission_control_new ();
-	status = mission_control_get_connection_status (mc, account, NULL);
-	g_object_unref (mc);
-	if (status != TP_CONNECTION_STATUS_CONNECTED) {
-		/* Account is disconnected */
+	connection = empathy_account_get_connection (account);
+	if (connection == NULL)
 		return FALSE;
-	}
 
-	profile = mc_account_get_profile (account);
-	protocol_name = mc_profile_get_protocol_name (profile);
-	if (strcmp (protocol_name, "local-xmpp") == 0) {
-		/* We can't add accounts to a XMPP LL connection
-		 * FIXME: We should inspect the flags of the contact list group interface
-		 */
-		g_object_unref (profile);
-		return FALSE;
-	}
+	contact_manager = empathy_contact_manager_dup_singleton ();
+	result = empathy_contact_manager_get_flags_for_connection (
+		contact_manager, connection) & EMPATHY_CONTACT_LIST_CAN_ADD;
+	g_object_unref (contact_manager);
 
-	g_object_unref (profile);
-	return TRUE;
+	return result;
 }
 
 static void
@@ -274,13 +355,12 @@ new_contact_response_cb (GtkDialog *dialog,
 	EmpathyContactManager *manager;
 	EmpathyContact         *contact;
 
-	manager = empathy_contact_manager_new ();
+	manager = empathy_contact_manager_dup_singleton ();
 	contact = empathy_contact_widget_get_contact (contact_widget);
 
 	if (contact && response == GTK_RESPONSE_OK) {
 		empathy_contact_list_add (EMPATHY_CONTACT_LIST (manager),
-					  contact,
-					  _("I would like to add you to my contact list."));
+					  contact, "");
 	}
 
 	new_contact_dialog = NULL;
@@ -290,6 +370,13 @@ new_contact_response_cb (GtkDialog *dialog,
 
 void
 empathy_new_contact_dialog_show (GtkWindow *parent)
+{
+	empathy_new_contact_dialog_show_with_contact (parent, NULL);
+}
+
+void
+empathy_new_contact_dialog_show_with_contact (GtkWindow *parent,
+                                              EmpathyContact *contact)
 {
 	GtkWidget *dialog;
 	GtkWidget *button;
@@ -313,7 +400,7 @@ empathy_new_contact_dialog_show (GtkWindow *parent)
 				      button,
 				      GTK_RESPONSE_CANCEL);
 	gtk_widget_show (button);
-	
+
 	/* Add button */
 	button = gtk_button_new_with_label (GTK_STOCK_ADD);
 	gtk_button_set_use_stock (GTK_BUTTON (button), TRUE);
@@ -323,17 +410,18 @@ empathy_new_contact_dialog_show (GtkWindow *parent)
 	gtk_widget_show (button);
 
 	/* Contact info widget */
-	contact_widget = empathy_contact_widget_new (NULL,
+	contact_widget = empathy_contact_widget_new (contact,
 						     EMPATHY_CONTACT_WIDGET_EDIT_ALIAS |
 						     EMPATHY_CONTACT_WIDGET_EDIT_ACCOUNT |
 						     EMPATHY_CONTACT_WIDGET_EDIT_ID |
 						     EMPATHY_CONTACT_WIDGET_EDIT_GROUPS);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 			    contact_widget,
 			    TRUE, TRUE, 0);
 	empathy_contact_widget_set_account_filter (contact_widget,
 						   can_add_contact_to_account,
 						   NULL);
+	gtk_widget_show (contact_widget);
 
 	new_contact_dialog = dialog;
 
