@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Mikael Hallendal <micke@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
@@ -68,6 +68,9 @@ void                     empathy_message_set_body          (EmpathyMessage
 time_t                   empathy_message_get_timestamp     (EmpathyMessage           *message);
 void                     empathy_message_set_timestamp     (EmpathyMessage           *message,
 							    time_t                    timestamp);
+gboolean                 empathy_message_is_backlog        (EmpathyMessage           *message);
+void                     empathy_message_set_is_backlog    (EmpathyMessage           *message,
+							    gboolean                 is_backlog);
 gboolean                 empathy_message_should_highlight  (EmpathyMessage           *message);
 TpChannelTextMessageType empathy_message_type_from_str     (const gchar              *type_str);
 const gchar *            empathy_message_type_to_str       (TpChannelTextMessageType  type);
