@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- * 
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
@@ -25,11 +25,16 @@
 
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/gtypes.h>
+#include <telepathy-glib/dbus.h>
+#if HAVE_GEOCLUE
+#include <geoclue/geoclue-geocode.h>
+#endif
 
 #include <extensions/extensions.h>
 
 #include "empathy-tp-contact-factory.h"
 #include "empathy-utils.h"
+#include "empathy-location.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_TP | EMPATHY_DEBUG_CONTACT
 #include "empathy-debug.h"
@@ -46,6 +51,7 @@ typedef struct {
 	guint           avatar_max_height;
 	guint           avatar_max_size;
 	gboolean        can_request_ft;
+	gboolean        can_request_st;
 } EmpathyTpContactFactoryPriv;
 
 G_DEFINE_TYPE (EmpathyTpContactFactory, empathy_tp_contact_factory, G_TYPE_OBJECT);
@@ -390,6 +396,177 @@ tp_contact_factory_got_capabilities (EmpathyTpContactFactory *tp_factory,
 	g_ptr_array_free (capabilities, TRUE);
 }
 
+#if HAVE_GEOCLUE
+#define GEOCODE_SERVICE "org.freedesktop.Geoclue.Providers.Yahoo"
+#define GEOCODE_PATH "/org/freedesktop/Geoclue/Providers/Yahoo"
+
+/* This callback is called by geoclue when it found a position
+ * for the given address.  A position is necessary for a contact
+ * to show up on the map
+ */
+static void
+geocode_cb (GeoclueGeocode *geocode,
+	    GeocluePositionFields fields,
+	    double latitude,
+	    double longitude,
+	    double altitude,
+	    GeoclueAccuracy *accuracy,
+	    GError *error,
+	    gpointer contact)
+{
+	GValue *new_value;
+	GHashTable *location;
+
+	location = empathy_contact_get_location (EMPATHY_CONTACT (contact));
+
+	if (error != NULL) {
+		DEBUG ("Error geocoding location : %s", error->message);
+		g_object_unref (geocode);
+		g_object_unref (contact);
+		return;
+	}
+
+	if (fields & GEOCLUE_POSITION_FIELDS_LATITUDE) {
+		new_value = tp_g_value_slice_new_double (latitude);
+		g_hash_table_replace (location, g_strdup (EMPATHY_LOCATION_LAT),
+			new_value);
+		DEBUG ("\t - Latitude: %f", latitude);
+	}
+	if (fields & GEOCLUE_POSITION_FIELDS_LONGITUDE) {
+		new_value = tp_g_value_slice_new_double (longitude);
+		g_hash_table_replace (location, g_strdup (EMPATHY_LOCATION_LON),
+			new_value);
+		DEBUG ("\t - Longitude: %f", longitude);
+	}
+	if (fields & GEOCLUE_POSITION_FIELDS_ALTITUDE) {
+		new_value = tp_g_value_slice_new_double (altitude);
+		g_hash_table_replace (location, g_strdup (EMPATHY_LOCATION_ALT),
+			new_value);
+		DEBUG ("\t - Altitude: %f", altitude);
+	}
+
+	/* Don't change the accuracy as we used an address to get this position */
+	g_object_notify (contact, "location");
+	g_object_unref (geocode);
+	g_object_unref (contact);
+}
+#endif
+
+#if HAVE_GEOCLUE
+static gchar *
+get_dup_string (GHashTable *location,
+    gchar *key)
+{
+  GValue *value;
+
+  value = g_hash_table_lookup (location, key);
+  if (value != NULL)
+    return g_value_dup_string (value);
+
+  return NULL;
+}
+#endif
+
+static void
+tp_contact_factory_geocode (EmpathyContact *contact)
+{
+#if HAVE_GEOCLUE
+	static GeoclueGeocode *geocode;
+	gchar *str;
+	GHashTable *address;
+	GValue* value;
+	GHashTable *location;
+
+	location = empathy_contact_get_location (contact);
+	if (location == NULL)
+		return;
+
+	value = g_hash_table_lookup (location, EMPATHY_LOCATION_LAT);
+	if (value != NULL)
+		return;
+
+	if (geocode == NULL) {
+		geocode = geoclue_geocode_new (GEOCODE_SERVICE, GEOCODE_PATH);
+		g_object_add_weak_pointer (G_OBJECT (geocode), (gpointer *) &geocode);
+	}
+	else
+		g_object_ref (geocode);
+
+	address = geoclue_address_details_new ();
+
+	str = get_dup_string (location, EMPATHY_LOCATION_COUNTRY_CODE);
+	if (str != NULL) {
+		g_hash_table_insert (address,
+			g_strdup (GEOCLUE_ADDRESS_KEY_COUNTRYCODE), str);
+		DEBUG ("\t - countrycode: %s", str);
+	}
+
+	str = get_dup_string (location, EMPATHY_LOCATION_COUNTRY);
+	if (str != NULL) {
+		g_hash_table_insert (address,
+			g_strdup (GEOCLUE_ADDRESS_KEY_COUNTRY), str);
+		DEBUG ("\t - country: %s", str);
+	}
+
+	str = get_dup_string (location, EMPATHY_LOCATION_POSTAL_CODE);
+	if (str != NULL) {
+		g_hash_table_insert (address,
+			g_strdup (GEOCLUE_ADDRESS_KEY_POSTALCODE), str);
+		DEBUG ("\t - postalcode: %s", str);
+	}
+
+	str = get_dup_string (location, EMPATHY_LOCATION_REGION);
+	if (str != NULL) {
+		g_hash_table_insert (address,
+			g_strdup (GEOCLUE_ADDRESS_KEY_REGION), str);
+		DEBUG ("\t - region: %s", str);
+	}
+
+	str = get_dup_string (location, EMPATHY_LOCATION_LOCALITY);
+	if (str != NULL) {
+		g_hash_table_insert (address,
+			g_strdup (GEOCLUE_ADDRESS_KEY_LOCALITY), str);
+		DEBUG ("\t - locality: %s", str);
+	}
+
+	str = get_dup_string (location, EMPATHY_LOCATION_STREET);
+	if (str != NULL) {
+		g_hash_table_insert (address,
+			g_strdup (GEOCLUE_ADDRESS_KEY_STREET), str);
+		DEBUG ("\t - street: %s", str);
+	}
+
+	g_object_ref (contact);
+	geoclue_geocode_address_to_position_async (geocode, address,
+		geocode_cb, contact);
+
+	g_hash_table_unref (address);
+#endif
+}
+
+static void
+tp_contact_factory_update_location (EmpathyTpContactFactory *tp_factory,
+				    guint handle,
+				    GHashTable *location)
+{
+	EmpathyContact *contact;
+	GHashTable     *new_location;
+
+	contact = tp_contact_factory_find_by_handle (tp_factory, handle);
+
+	if (contact == NULL)
+		return;
+
+	new_location = g_hash_table_new_full (g_str_hash, g_str_equal,
+		(GDestroyNotify) g_free, (GDestroyNotify) tp_g_value_slice_free);
+	tp_g_hash_table_update (new_location, location, (GBoxedCopyFunc) g_strdup,
+		(GBoxedCopyFunc) tp_g_value_slice_dup);
+	empathy_contact_set_location (contact, new_location);
+	g_hash_table_unref (new_location);
+
+	tp_contact_factory_geocode (contact);
+}
+
 static void
 tp_contact_factory_got_locations (TpProxy                 *tp_proxy,
 				  GHashTable              *locations,
@@ -411,12 +588,8 @@ tp_contact_factory_got_locations (TpProxy                 *tp_proxy,
 	while (g_hash_table_iter_next (&iter, &key, &value)) {
 		guint           handle = GPOINTER_TO_INT (key);
 		GHashTable     *location = value;
-		EmpathyContact *contact;
 
-		contact = tp_contact_factory_find_by_handle (tp_factory, handle);
-		if (contact != NULL) {
-			empathy_contact_set_location (contact, location);
-		}
+		tp_contact_factory_update_location (tp_factory, handle, location);
 	}
 }
 
@@ -458,14 +631,7 @@ tp_contact_factory_location_updated_cb (TpProxy      *proxy,
 					GObject      *weak_object)
 {
 	EmpathyTpContactFactory *tp_factory = EMPATHY_TP_CONTACT_FACTORY (weak_object);
-	EmpathyContact          *contact;
-
-	contact = tp_contact_factory_find_by_handle (tp_factory, handle);
-
-	if (contact == NULL)
-		return;
-
-	empathy_contact_set_location (contact, location);
+	tp_contact_factory_update_location (tp_factory, handle, location);
 }
 
 static void
@@ -479,6 +645,7 @@ get_requestable_channel_classes_cb (TpProxy *connection,
 	EmpathyTpContactFactoryPriv *priv = GET_PRIV (self);
 	GPtrArray                   *classes;
 	guint                        i;
+	GList                       *l;
 
 	if (error != NULL) {
 		DEBUG ("Error: %s", error->message);
@@ -490,39 +657,46 @@ get_requestable_channel_classes_cb (TpProxy *connection,
 		GValueArray *class_struct;
 		GHashTable *fixed_prop;
 		GValue *chan_type, *handle_type;
-		GList *l;
 
 		class_struct = g_ptr_array_index (classes, i);
 		fixed_prop = g_value_get_boxed (g_value_array_get_nth (class_struct, 0));
 
-		chan_type = g_hash_table_lookup (fixed_prop,
-			TP_IFACE_CHANNEL ".ChannelType");
-		if (chan_type == NULL ||
-		    tp_strdiff (g_value_get_string (chan_type),
-		    		TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER)) {
-			continue;
-		}
-
 		handle_type = g_hash_table_lookup (fixed_prop,
 			TP_IFACE_CHANNEL ".TargetHandleType");
 		if (handle_type == NULL ||
-		    g_value_get_uint (handle_type) != TP_HANDLE_TYPE_CONTACT) {
+		    g_value_get_uint (handle_type) != TP_HANDLE_TYPE_CONTACT)
 			continue;
-		}
 
-		/* We can request file transfer channel to contacts. */
-		priv->can_request_ft = TRUE;
+		chan_type = g_hash_table_lookup (fixed_prop,
+			TP_IFACE_CHANNEL ".ChannelType");
+		if (chan_type == NULL)
+			continue;
 
-		/* Update the capabilities of all contacts */
-		for (l = priv->contacts; l != NULL; l = g_list_next (l)) {
-			EmpathyContact *contact = l->data;
-			EmpathyCapabilities caps;
+		if (!tp_strdiff (g_value_get_string (chan_type),
+		    TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
+			priv->can_request_ft = TRUE;
+		else if (!tp_strdiff (g_value_get_string (chan_type),
+		         TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+			priv->can_request_st = TRUE;
+	}
 
-			caps = empathy_contact_get_capabilities (contact);
-			empathy_contact_set_capabilities (contact, caps |
-				EMPATHY_CAPABILITIES_FT);
-		}
-		break;
+	if (!priv->can_request_ft && !priv->can_request_st)
+		return ;
+
+	/* Update the capabilities of all contacts */
+	for (l = priv->contacts; l != NULL; l = g_list_next (l)) {
+		EmpathyContact *contact = l->data;
+		EmpathyCapabilities caps;
+
+		caps = empathy_contact_get_capabilities (contact);
+
+		if (priv->can_request_ft)
+			caps |= EMPATHY_CAPABILITIES_FT;
+
+		if (priv->can_request_st)
+			caps |= EMPATHY_CAPABILITIES_STREAM_TUBE;
+
+		empathy_contact_set_capabilities (contact, caps);
 	}
 }
 
@@ -563,10 +737,11 @@ tp_contact_factory_add_contact (EmpathyTpContactFactory *tp_factory,
 	EmpathyTpContactFactoryPriv *priv = GET_PRIV (tp_factory);
 	TpHandle self_handle;
 	TpHandle handle;
-	GArray handles = {(gchar*) &handle, 1};
+	GArray handles = {(gchar *) &handle, 1};
 	GHashTable *tokens;
 	GPtrArray *capabilities;
 	GError *error = NULL;
+	EmpathyCapabilities caps;
 
 	/* Keep a weak ref to that contact */
 	g_object_weak_ref (G_OBJECT (contact),
@@ -579,16 +754,20 @@ tp_contact_factory_add_contact (EmpathyTpContactFactory *tp_factory,
 				g_object_ref (tp_factory),
 				g_object_unref);
 
+	caps = empathy_contact_get_capabilities (contact);
+
 	/* Set the FT capability */
 	if (priv->can_request_ft) {
-		EmpathyCapabilities caps;
-
-		caps = empathy_contact_get_capabilities (contact);
 		caps |= EMPATHY_CAPABILITIES_FT;
-
-		empathy_contact_set_capabilities (contact, caps);
 	}
 
+	/* Set the Stream Tube capability */
+	if (priv->can_request_st) {
+		caps |= EMPATHY_CAPABILITIES_STREAM_TUBE;
+	}
+
+	empathy_contact_set_capabilities (contact, caps);
+
 	/* Set is-user property. Note that it could still be the handle is
 	 * different from the connection's self handle, in the case the handle
 	 * comes from a group interface. */
@@ -988,7 +1167,7 @@ empathy_tp_contact_factory_set_avatar (EmpathyTpContactFactory *tp_factory,
 	if (data && size > 0 && size < G_MAXUINT) {
 		GArray avatar;
 
-		avatar.data = (gchar*) data;
+		avatar.data = (gchar *) data;
 		avatar.len = size;
 
 		DEBUG ("Setting avatar on connection %s",
@@ -1252,6 +1431,7 @@ empathy_tp_contact_factory_init (EmpathyTpContactFactory *tp_factory)
 
 	tp_factory->priv = priv;
 	priv->can_request_ft = FALSE;
+	priv->can_request_st = FALSE;
 }
 
 static GHashTable *factories = NULL;
